/*!=============================================================================
	Copyright (c) 2003 pmG WorldWide, LLC.

	\file Dot.c
	\author Ron Griswold
	
	Created: 9/11/03

	CVS: $Revision$	$Date$

	Description: Dot Utility Shader

=============================================================================*/
#include "Plugin.h"

//////////////////////////////////////////////////////
// Globals
//////////////////////////////////////////////////////

// Data Keys
static int		DATA_KEYID		= 0;					// fxData* keyID
static int *	DATA_KEY		= &DATA_KEYID;			// fxData* key

// Shader Input IDs
static FXint	IN_V1			= 0;					
static FXint	IN_V2			= 0;

// Shader Output IDs
static FXint	OUT_VAL			= 0;

//////////////////////////////////////////////////////
// State Data
//////////////////////////////////////////////////////
typedef struct
{
	FX_Vector	v1, v2;

}State;

static int	StateInit( State *s )
{
	if( !s ) return FX_FALSE;

	memset( &(s->v1), 0, sizeof(FX_Vector) );
	memset( &(s->v2), 0, sizeof(FX_Vector) );

	return FX_TRUE;
};

//////////////////////////////////////////////////////
// Shader Callbacks
//////////////////////////////////////////////////////
static FX_SHADERINITINPUTS(ShaderInitInputs)
// FXvoidf f(FXshader shader, FX_ShaderGlobals *shader_globals, FX_ShaderParam *input)
{
	State *state = (State *)fxShaderGetTypeData( shader, FX_NOFLAG );
	if( !state ) return;

	memset( &(input[IN_V1].vec), 0, sizeof(FX_Vector) );
	memset( &(input[IN_V2].vec), 0, sizeof(FX_Vector) );
}

static FX_SHADEREVALUATE(ShaderEvaluate)
// FXvoidf f(FXshader shader, FX_ShaderGlobals *shader_globals, FX_ShaderParam *input, FX_ShaderParam *output)
{
	State *state = (State *)fxShaderGetTypeData( shader, FX_NOFLAG );
	if( !state ) return;

	inVal = input[IN_VAL].f;

	// If there's an input connected to the node, use its value for the output, otherwise use the keyed value
	if( inVal != 0.0 )
		output[OUT_VAL].f = inVal;
	else
        output[OUT_VAL].f = state->val;
}

//////////////////////////////////////////////////////
// Control Callbacks
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// Access Function
//////////////////////////////////////////////////////
FX_ACCESSFUNC( Dot_Access )
// FXintf f(FX_AccessInfo *ai, FXentity ID, FXint level, FXint64 entry)
{
	FXshader	shader = (FXshader)ID;
	State *		state = NULL;

	switch( level )
	{
	case ACCESS_OBJECT:
		{
			switch( entry )
			{
			case O_CREATE:
				{
					// Create State Data
					if( !( state = (State *)fxDataCreate( shader, DATA_KEY, DATA_KEYID, sizeof( State ))) )
					{
						//Send Error Message
						fxMessageSend(FX_NULLID, "Dot - Utility Shader", FX_MESSAGE_POST|FX_MESSAGE_ERROR,
							"Couldn't allocate necessary object data!", NULL);

						return CONTINUE_ENTRY;
					}
				
					// Initialize State Data
					StateInit( state );

					// Set State Data (Necessary for proper shader evaluation)
					fxObjectSetTypeData( shader, state, FX_NOFLAG );

					// Setup Channels
					fxChannelSetup( shader, "val", FX_CHAN_0, FX_CHANTYPE_VALUE, (FXdouble)state->val, 0.0, 0.0, FX_NOFLAG );

					// Link Channels to State Data
					fxChannelDataLink( shader, FX_CHAN_0, FX_ARG_DOUBLE, &state->val, FX_NOFLAG );

					// Create Inputs
					IN_VAL		= fxShaderCreateInput( shader, "Value", FX_IOTYPE_FLOAT, FX_NOFLAG );
					
					// Create Outputs
					OUT_VAL		= fxShaderCreateOutput( shader, "Value", FX_IOTYPE_FLOAT, FX_NOFLAG );

					// Set Shader Callbacks
					fxShaderCallback( shader, FX_SHADERFUNC_INITINPUTS, &ShaderInitInputs );
					fxShaderCallback( shader, FX_SHADERFUNC_EVALUATE, &ShaderEvaluate );

				} // O_CREATE
				break;

			case O_DESTROY:
				{

				} // O_DESTROY
				break;
			}
		} // ACCESS_OBJECT
		break;

	case ACCESS_INTERFACE:
		{
			switch( entry )
			{
			case IN_CREATE:
				{
					// Controls
					FXcontrol		MainBlock		= FX_NULLID;
					FXcontrol		ValueCtl		= FX_NULLID;

					// Create the Controls
					MainBlock = fxCC_Block( "Dot", NULL, ACC_NONE, FX_NOFLAG );

						ValueCtl = fxCC_ShaderChannelFloat( "Value", NULL, ACC_NONE, FX_CHAN_0, IN_VAL, FX_NOFLAG );
						fxConSetPos( ValueCtl, 100, 4 );

					//Assign the Blocks
					fxInterfaceBlockSet(FX_MODE_RENDER, FX_MODE_RENDER, FX_MBLOCK_SHADER, FX_NULLID);
					fxInterfaceBlockSet(FX_MODE_RENDER, NULL, NULL, MainBlock );
					fxInterfaceBlockSet(FX_MODE_RENDER, FX_MODE_RENDER, FX_MBLOCK_SPLINE, FX_NULLID);

				} // O_CREATE
				break;
			}
		} // ACCESS_INTERFACE
		break;
	}

	return ENTRY_OK;
}