/*=============================================================================
	Copyright (c) 1999-2003 pmG Worldwide, LLC.

	File:	messiah_expfunc.h
	Author:	Lyle Milton
	Date:	03/27/03

	-----------------------------------------------------------------

	These are the same functions that would be used in expressions
	& scripts.  While you could use the relevant fxExpCompute* function
	to compute an expression, a number of the expression functions are
	published here soley for speed & convenience.  Follow any notes 
	and/or descriptions you'd find in Command mode or function_list.txt.

=============================================================================*/


#ifdef _MESSIAH_API_EXPFUNC

#ifndef __MESSIAH_EXPFUNC
#define __MESSIAH_EXPFUNC

#ifndef	__MESSIAH_HIDE_TYPES


	#define xpos						FX_X_POSITION
	#define ypos						FX_Y_POSITION
	#define zpos						FX_Z_POSITION
	#define heading						FX_HEADING
	#define pitch						FX_PITCH
	#define bank						FX_BANK
	#define xscale						FX_X_SCALE
	#define yscale						FX_Y_SCALE
	#define zscale						FX_Z_SCALE
	#define xpivot						FX_X_PIVOT
	#define ypivot						FX_Y_PIVOT
	#define zpivot						FX_Z_PIVOT


	#define PARTICLE_DEAD			(1<<0)
	#define PARTICLE_ALIVE			(1<<1)
	#define PARTICLE_NEW				(1<<2)

#endif	//__MESSIAH_HIDE_TYPES



//vector lib
FX_API(FXdouble, Angle)(FXobject o, FXdouble frame);

FX_API(FXdouble, AngleW)(FXobject o, FXdouble frame);

FX_API(FXdouble, AngleBetween)(FXobject o0, FXobject o1, FXdouble frame0, FXdouble frame1);

FX_API(FXdouble, AngleBetweenW)(FXobject o0, FXobject o1, FXdouble frame0, FXdouble frame1);

FX_API(FXdouble, AngleBetweenTri)(FXobject o0, FXobject o1, FXobject o2, FXdouble frame0, FXdouble frame1, FXdouble frame2);

FX_API(FXdouble, WeightValue)(FXobject o, FXdouble x, FXdouble y, FXdouble z);


//control lib
FX_API(FXdouble, Follow)(FXobject d, FXobject s, FXint channel, FXdouble distance);

FX_API(FXdouble, FollowFrame)(FXobject d, FXobject s, FXint channel, FXdouble frame);

FX_API(FXdouble, Cycle)(FXobject d, FXint d_channel, FXobject s, FXint s_channel, FXdouble factor);

FX_API(FXdouble, CycleParam)(FXobject s, FXint s_channel, FXdouble val);

FX_API(FXdouble, CycleFrame)(FXobject s, FXint s_channel, FXdouble val);

FX_API(FXdouble, CycleBranchFrame)(FXobject s, FXint s_channel, FXdouble val);

FX_API(FXdouble, CycleBranchParam)(FXobject s, FXint s_channel, FXdouble val);

FX_API(FXdouble, CycleBranchMorphFrame)(FXobject d, FXobject s, FXint s_channel, FXdouble frame, FXdouble u);

FX_API(FXdouble, CycleBranchMorphParam)(FXobject d, FXobject s, FXint s_channel, FXdouble frame, FXdouble u);

FX_API(FXdouble, CycleBranchMorphAddFrame)(FXobject d, FXobject s, FXint s_channel, FXdouble frame, FXdouble u);

FX_API(FXdouble, CycleBranchMorphAddParam)(FXobject d, FXobject s, FXint s_channel, FXdouble frame, FXdouble u);

FX_API(FXdouble, InheritRoot)(FXobject s, FXobject d, FXint channel, FXdouble frame, FXdouble factor);

FX_API(FXdouble, InheritEnd)(FXobject s, FXobject d, FXint channel, FXdouble frame, FXdouble factor);

FX_API(FXdouble, ChainWave)(FXobject s, FXobject d, FXint channel, FXdouble wave, FXdouble curve, FXdouble oscillate);

FX_API(FXdouble, ChainTwist)(FXobject s, FXobject d, FXdouble start_bank, FXdouble end_bank, FXint flags);

FX_API(FXdouble, MirrorRadial)(FXobject s, FXdouble frame, FXint compute_pitch);

FX_API(FXdouble, DynamicParent)(FXobject s1, FXobject s2, FXobject obj, FXdouble u, FXdouble v);

FX_API(FXdouble, AlignLinearRel)(FXobject s1, FXobject s2, FXobject obj, FXdouble u, FXdouble v);

FX_API(FXdouble, AlignLinearAbs)(FXobject s1, FXobject s2, FXobject obj, FXdouble u, FXdouble v);

FX_API(FXdouble, AlignTriAbs)(FXobject s1, FXobject s2, FXobject s3, FXobject obj, FXdouble u, FXdouble v, FXdouble m, FXdouble r);

FX_API(FXdouble, AlignTriRel)(FXobject s1, FXobject s2, FXobject s3, FXobject obj, FXdouble u, FXdouble v, FXdouble m, FXdouble r);

FX_API(FXdouble, AlignQuadAbs)(FXobject s1, FXobject s2, FXobject s3, FXobject s4, FXobject obj, FXdouble u, FXdouble v, FXdouble m, FXdouble r);

FX_API(FXdouble, AlignQuadRel)(FXobject s1, FXobject s2, FXobject s3, FXobject s4, FXobject obj, FXdouble u, FXdouble v, FXdouble m, FXdouble r);


//speed lib
FX_API(FXdouble, Speed)(FXobject d, FXdouble time,FXdouble interval);

FX_API(FXdouble, Acceleration)(FXobject d, FXdouble time,FXdouble interval);

FX_API(FXdouble, AccelerationSum)(FXobject d, FXdouble start_time,FXdouble end_time,FXdouble interval);



//path lib
FX_API(FXdouble, PathLength)(FXobject d);

FX_API(FXdouble, PathSegment)(FXobject d, FXdouble s_frame, FXdouble d_frame);

FX_API(FXdouble, PathSoFar)(FXobject d, FXdouble d_frame);

FX_API(FXdouble, PathToGo)(FXobject d, FXdouble d_frame);

FX_API(FXdouble, PathFramesToGo)(FXobject d, FXdouble d_frame);

FX_API(FXdouble, PathFrames)(FXobject d);

FX_API(FXdouble, PathLengthToFrame)(FXobject d, FXdouble length);


//motion lib
FX_API(FXint,	MotionBlend)(FXobject source, FXobject target, FXdouble startframe, FXdouble endframe, FXint channels, FXdouble percent);

FX_API(FXint,	KeyFader)(FXobject obj, FXdouble startframe, FXdouble endframe, FXint channels, FXdouble percent);

FX_API(FXint,	CopyChan)(FXobject d, FXobject s, FXchar *chan_bits, FXchar *add_bits);


//constraint lib
FX_API(FXint,	KneeTangent)(FXobject root, FXobject knee, FXvecd tangent, FXint flags); //API ONLY

FX_API(FXint,	KneeConstraint)(FXobject root, FXobject knee, FXobject goal, FXint channel);

FX_API(FXint,	Target)(FXobject endeff, FXobject goal);

FX_API(FXint,	TargetTime)(FXobject endeff, FXobject goal,FXdouble gtime, FXint blend);

FX_API(FXint,	LockedGoal)(FXobject goal, FXobject endeff,FXint on);

FX_API(FXint,	MoveTo)(FXobject endeff, FXobject goal,FXint on);

FX_API(FXdouble,	RelativeTo)(FXobject obj, FXobject coord_sys, FXint channel);

FX_API(FXint,	LockToGoal)(FXobject root, FXobject endeff, FXobject goal,FXint on);

FX_API(FXint,	Align)(FXobject endeff, FXobject goal,FXint on);


//muscle lib
FX_API(FXint,	Muscle)(FXobject muscle, FXobject connection, FXdouble tension, FXdouble contraction);

FX_API(FXint,	TargetStretch)(FXobject obj, FXobject target, FXint flags);

//ik lib
FX_API(FXint,	IK2D)(FXobject root, FXobject endeff, FXobject goal,FXint align,FXint on);

FX_API(FXint,	IKTel)(FXobject root, FXobject endeff, FXobject goal,FXint align,FXint on);

FX_API(FXint,	IKAlign)(FXobject root, FXobject endeff, FXobject goal, FXint on);

FX_API(FXint,	IKStiff)(FXobject joint, FXint channel, FXint stiffness);



//particles lib
FX_API(FXint, ParticleSetFDense)(FXeffect, FXdouble value);

FX_API(FXint, ParticleSetDamping)(FXeffect, FXdouble value);

FX_API(FXint, ParticleSetGWeight)(FXeffect, FXdouble value);

FX_API(FXint, ParticleSetSTime)(FXeffect, FXdouble value);

FX_API(FXint, ParticleSetPartsPerSec)(FXeffect, FXint value);

FX_API(FXint, ParticleSetSamples)(FXeffect, FXint value);

FX_API(FXint, ParticleSetIPressure)(FXeffect, FXdouble value);

FX_API(FXint, ParticleGetPos)(FXeffect, FXint pindex, FXvecd pos);

FX_API(FXint, ParticleGetDispVec)(FXeffect, FXint pindex, FXvecd vec);

FX_API(FXint, ParticleGetLocalAxis)(FXeffect, FXint pindex, FXint axis, FXdouble vec[3]);

FX_API(FXint, ParticleGetState)(FXeffect, FXint pindex, FXint *state);

FX_API(FXint, ParticleGetAttribute)(FXeffect, FXint pindex, FXint attribute, FXfloat *value);

//100505
FX_API(FXint, ParticleGetPrevPos)(FXeffect, FXint pindex, FXvecd vec);
FX_API(FXint, ParticleGetVelocity)(FXeffect, FXint pindex, FXvecd vec);
FX_API(FXint, ParticleGetNumActive)(FXeffect, FXint *num);
FX_API(FXint, ParticleGetNum)(FXeffect, FXint *num);	

FX_API(FXint, ParticleSetPos)(FXeffect, FXint pindex, FXvecd pos);
FX_API(FXint, ParticleSetPrevPos)(FXeffect, FXint pindex, FXvecd vec);
FX_API(FXint, ParticleSetVelocity)(FXeffect, FXint pindex, FXvecd vec);
FX_API(FXint, ParticleSetState)(FXeffect, FXint pindex, FXint *state);
FX_API(FXint, ParticleSetLocalAxis)(FXeffect, FXint pindex, FXint axis, FXvecd vec);

FX_API(FXint, ParticleSetLocalAxis3)(FXeffect, FXint pindex, FXvecd xvec,FXvecd yvec,FXvecd zvec);
FX_API(FXint, ParticleGetLocalAxis3)(FXeffect, FXint pindex, FXvecd xvec,FXvecd yvec,FXvecd zvec);

FX_API(FXint, ParticleGetRotationVelocity)(FXeffect, FXint pindex, FXvecd hpb);
FX_API(FXint, ParticleSetRotationVelocity)(FXeffect, FXint pindex, FXvecd hpb);

FX_API(FXint, ParticleSetAttribute)(FXeffect, FXint pindex, FXint attribute, FXfloat *value);

//metanurb lib
FX_API(FXint, MetaNURBGetSubPoints)(FXobject objid, FXint polid,FXint sub_level,FXint *num_points, FXvecd *points, FXvecd *normals);

/////////////////////////


#define _MESSIAH_PTR_EXPFUNC	messiah_apicomponent = FX_APIFUNCS_EXPFUNC;\
								FX_PTR(Angle);\
								FX_PTR(AngleW);\
								FX_PTR(AngleBetween);\
								FX_PTR(AngleBetweenW);\
								FX_PTR(AngleBetweenTri);\
								FX_PTR(WeightValue);\
								\
								FX_PTR(Follow);\
								FX_PTR(FollowFrame);\
								FX_PTR(Cycle);\
								FX_PTR(CycleParam);\
								FX_PTR(CycleFrame);\
								FX_PTR(CycleBranchFrame);\
								FX_PTR(CycleBranchParam);\
								FX_PTR(CycleBranchMorphFrame);\
								FX_PTR(CycleBranchMorphParam);\
								FX_PTR(CycleBranchMorphAddFrame);\
								FX_PTR(CycleBranchMorphAddParam);\
								FX_PTR(InheritRoot);\
								FX_PTR(InheritEnd);\
								FX_PTR(ChainWave);\
								FX_PTR(ChainTwist);\
								FX_PTR(MirrorRadial);\
								FX_PTR(DynamicParent);\
								FX_PTR(AlignLinearRel);\
								FX_PTR(AlignLinearAbs);\
								FX_PTR(AlignTriAbs);\
								FX_PTR(AlignTriRel);\
								FX_PTR(AlignQuadAbs);\
								FX_PTR(AlignQuadRel);\
								\
								FX_PTR(Speed);\
								FX_PTR(Acceleration);\
								FX_PTR(AccelerationSum);\
								\
								FX_PTR(PathLength);\
								FX_PTR(PathSegment);\
								FX_PTR(PathSoFar);\
								FX_PTR(PathToGo);\
								FX_PTR(PathFramesToGo);\
								FX_PTR(PathFrames);\
								FX_PTR(PathLengthToFrame);\
								\
								FX_PTR(MotionBlend);\
								FX_PTR(KeyFader);\
								FX_PTR(CopyChan);\
								\
								FX_PTR(KneeTangent);\
								FX_PTR(KneeConstraint);\
								FX_PTR(Target);\
								FX_PTR(TargetTime);\
								FX_PTR(LockedGoal);\
								FX_PTR(MoveTo);\
								FX_PTR(RelativeTo);\
								FX_PTR(LockToGoal);\
								FX_PTR(Align);\
								\
								FX_PTR(Muscle);\
								FX_PTR(TargetStretch);\
								\
								FX_PTR(IK2D);\
								FX_PTR(IKTel);\
								FX_PTR(IKAlign);\
								FX_PTR(IKStiff);\
								\
								FX_PTR(ParticleSetFDense);\
								FX_PTR(ParticleSetDamping);\
								FX_PTR(ParticleSetGWeight);\
								FX_PTR(ParticleSetSTime);\
								FX_PTR(ParticleSetPartsPerSec);\
								FX_PTR(ParticleSetSamples);\
								FX_PTR(ParticleSetIPressure);\
								FX_PTR(ParticleGetPos);\
								FX_PTR(ParticleGetDispVec);\
								FX_PTR(ParticleGetState);\
								FX_PTR(ParticleGetAttribute);\
								FX_PTR(ParticleGetLocalAxis);\
								\
								FX_PTR(ParticleGetPrevPos);\
								FX_PTR(ParticleGetVelocity);\
								FX_PTR(ParticleGetNumActive);\
								FX_PTR(ParticleGetNum);\
								FX_PTR(ParticleSetPos);\
								FX_PTR(ParticleSetPrevPos);\
								FX_PTR(ParticleSetVelocity);\
								FX_PTR(ParticleSetState);\
								FX_PTR(ParticleSetLocalAxis);\
								FX_PTR(ParticleSetLocalAxis3);\
								FX_PTR(ParticleGetLocalAxis3);\
								FX_PTR(ParticleGetRotationVelocity);\
								FX_PTR(ParticleSetRotationVelocity);\
								FX_PTR(ParticleSetAttribute);\
								FX_PTR(MetaNURBGetSubPoints)


#endif //__MESSIAH_EXPFUNC

#endif //_MESSIAH_API_EXPFUNC












