/*=============================================================================
	Copyright (c) 1999-2004 pmG Worldwide, LLC.

	File:	messiah_math.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 4/06/04 - LM

=============================================================================*/

#ifndef __MESSIAH_MATH
#define __MESSIAH_MATH


static FXint	__mTempvar_i = 0;
static FXdouble	__mTempvar_d = 0;
static FXfloat	__mTempvar_f = 0;

#ifndef PI
	#define PI						(3.1415926535898)
#endif
#ifndef TWOPI
	#define TWOPI					(6.2831853071796)
#endif
#ifndef HALFPI
	#define HALFPI					(1.570796326795)
#endif

#define RPD							(0.0174532925199)
#define DPR							(57.2957795130823)


#define FX_RADIANS(_d)				((_d)*RPD)
#define FX_DEGREES(_r)				((_r)*DPR)


#define FX_ABS(_a)					((_a)< 0 ? -(_a):(_a))
#define FX_MIN(_a, _b)				((_a)<(_b)?(_a):(_b))
#define FX_MAX(_a, _b)				((_a)>(_b)?(_a):(_b))
#define FX_CLAMP(_a, _b, _c)		(FX_MAX(FX_MIN((_a),(_c)),(_b)))
#define FX_FLOOR(_a)				((int)(_a)-((_a)< 0 &&(_a)!=(int)(_a)))  
#define FX_CIEL(_a)					((int)(_a)+((_a)> 0 &&(_a)!=(int)(_a)))   
#define FX_SIGN(_a)					(((_a)<0) ? -1 : 1)
#define FX_SQUARE(_a)				((_a)*(_a))
#define FX_LERP(_t, _a, _b)			((_a) + ((_b)-(_a))*(_t))
#define FX_HERP(_t, _a, _b)			((_a) + ((_b)-(_a))*(_t)*(_t)*(3.0-(_t)-(_t)))
#define FX_SWAP(_a, _b, _temp)		((_temp)=(_a), (_a)=(_b), (_b)=(_temp))

#define FX_RANDOM_01				((double)rand()/(FX_RAND_MAX+1)) //using max+1 to prevent various programming problems.  So, the param will never actually reach 1.0... but very close.
#define FX_RANDOM_11				(FX_RANDOM_01*2.0-1.0)
#define FX_RANDOM_INT(_max_n)		(FX_RANDOM_01*(_max_n))
#define FX_RANDOM_RANGE(_lo, _hi)	(FX_RANDOM_01*((_hi)-(_lo)) + (_lo)))
#define FX_RANDOM_BINARY			((rand()<FX_RAND_MAX_HALF)?0:1)


#define FX_V3SET1(_a, _s)\
	( (_a)[0] = (_s),\
	  (_a)[1] = (_s),\
	  (_a)[2] = (_s) )

#define FX_V3SET(_a, _b, _c, _d) \
	( (_a)[0]=(_b),\
	  (_a)[1]=(_c),\
	  (_a)[2]=(_d) )

#define FX_V3CLEAR(v)	FX_V3SET(v, 0, 0, 0)

#define FX_V3COPY(_a, _b) \
	( (_a)[0]=(_b)[0],\
	  (_a)[1]=(_b)[1],\
	  (_a)[2]=(_b)[2] )

#define FX_V3ADD1(_a, _b)\
	( (_a)[0] += (_b)[0],\
	  (_a)[1] += (_b)[1],\
	  (_a)[2] += (_b)[2] )

#define FX_V3ADD(_a, _b, _c) \
	( (_a)[0]=(_b)[0]+(_c)[0],\
	  (_a)[1]=(_b)[1]+(_c)[1],\
	  (_a)[2]=(_b)[2]+(_c)[2] )

#define FX_V3ADDS1(_a, _s)\
	( (_a)[0] += (_s),\
	  (_a)[1] += (_s),\
	  (_a)[2] += (_s) )

#define FX_V3ADDS(_a, _b, _s)\
	( (_a)[0] = (_b)[0] + (_s),\
	  (_a)[1] = (_b)[1] + (_s),\
	  (_a)[2] = (_b)[2] + (_s) )

#define FX_V3SUB1(_a, _b)\
	( (_a)[0] -= (_b)[0],\
	  (_a)[1] -= (_b)[1],\
	  (_a)[2] -= (_b)[2] )

#define FX_V3SUB(_a, _b, _c) \
	( (_a)[0]=(_b)[0]-(_c)[0],\
	  (_a)[1]=(_b)[1]-(_c)[1],\
	  (_a)[2]=(_b)[2]-(_c)[2] )

#define FX_V3SUBS1(_a, _s)\
	( (_a)[0] -= (_s),\
	  (_a)[1] -= (_s),\
	  (_a)[2] -= (_s) )

#define FX_V3SUBS(_a, _b, _s)\
	( (_a)[0] = (_b)[0] - (_s),\
	  (_a)[1] = (_b)[1] - (_s),\
	  (_a)[2] = (_b)[2] - (_s) )

#define FX_V3NEG1(_a) \
	( (_a)[0]=-(_a)[0],\
	  (_a)[1]=-(_a)[1],\
	  (_a)[2]=-(_a)[2] )

#define FX_V3NEG(_a, _b) \
	( (_a)[0]=-(_b)[0],\
	  (_a)[1]=-(_b)[1],\
	  (_a)[2]=-(_b)[2] )

#define FX_V3MULT1(_a, _b) \
	( (_a)[0]*=(_b)[0],\
	  (_a)[1]*=(_b)[1],\
	  (_a)[2]*=(_b)[2] )

#define FX_V3MULT(_a, _b, _c) \
	( (_a)[0]=(_b)[0]*(_c)[0],\
	  (_a)[1]=(_b)[1]*(_c)[1],\
	  (_a)[2]=(_b)[2]*(_c)[2] )

#define FX_V3SCALE1(_a, _b) \
	( (_a)[0]*=(_b),\
	  (_a)[1]*=(_b),\
	  (_a)[2]*=(_b) )

#define FX_V3SCALE(_a, _b, _c) \
	( (_a)[0]=(_b)[0]*_c,\
	  (_a)[1]=(_b)[1]*_c,\
	  (_a)[2]=(_b)[2]*_c )

#define FX_V3MID(_a, _b, _c) \
	( (_a)[0]=((_b)[0]+(_c)[0])*0.5,\
	  (_a)[1]=((_b)[1]+(_c)[1])*0.5,\
	  (_a)[2]=((_b)[2]+(_c)[2])*0.5 )

#define FX_V3MIN(_a, _b, _c)    \
	( (_a)[0]=FX_MIN((_b)[0],(_c)[0]),\
	  (_a)[1]=FX_MIN((_b)[1],(_c)[1]),\
	  (_a)[2]=FX_MIN((_b)[2],(_c)[2]), )

#define FX_V3MAX(_a, _b, _c)    \
	( (_a)[0]=FX_MAX((_b)[0],(_c)[0]),\
	  (_a)[1]=FX_MAX((_b)[1],(_c)[1]),\
	  (_a)[2]=FX_MAX((_b)[2],(_c)[2]) )

//clamp b within c & d................................
#define FX_V3CLAMP(_a, _b, _c, _d)    \
	( (_a)[0]=min(max((_b)[0],(_c)[0]), (_d)[0]),\
	  (_a)[1]=min(max((_b)[1],(_c)[1]), (_d)[1]),\
	  (_a)[2]=min(max((_b)[2],(_c)[2]), (_d)[2]) )

#define FX_V3EQUAL(_a, _b) \
	( ((_a)[0]==(_b)[0]) && ((_a)[1]==(_b)[1]) && ((_a)[2]==(_b)[2]) )

#define FX_V3SWAP(_a, _b, _temp)	\
	( FX_SWAP((_a)[0], (_b)[0], _temp),\
	  FX_SWAP((_a)[1], (_b)[1], _temp),\
	  FX_SWAP((_a)[2], (_b)[2], _temp) )

#define FX_V3DOT(_a, _b) \
	( (_a)[0]*(_b)[0]+(_a)[1]*(_b)[1]+(_a)[2]*(_b)[2] )

#define FX_V3CROSS(_a,_b,_c)\
    ( (_a)[0]=(_b)[1]*(_c)[2]-(_b)[2]*(_c)[1],\
	  (_a)[1]=(_b)[2]*(_c)[0]-(_b)[0]*(_c)[2],\
      (_a)[2]=(_b)[0]*(_c)[1]-(_b)[1]*(_c)[0] )

// Project vector _b onto vector _c (parallel component of _b with respect to _c)
#define FX_V3PROJ(_a,_b,_c)  (FX_V3COPY((_a),(_c)), FX_V3SCALE1((_a),FX_V3DOT((_b),(_c))/FX_V3DOT((_c),(_c))))

// NOTE: Perpendicular component of _b with respect to _c is _b minus the projection of _b onto _c
// FX_V3PROJ( proj, vec1, vec2 );
// FX_V3SUB( perp, vec1, proj );
#define FX_V3PERP( _p, _a, _b, _c)  (FX_V3PROJ( _a, _b, _c ), FX_V3SUB( _p, _b, _a ))

#define FX_V3DIST(_a, _b)	(sqrt(FX_SQUARE((_a)[0]-(_b)[0])+FX_SQUARE((_a)[1]-(_b)[1])+FX_SQUARE((_a)[2]-(_b)[2])))

#define FX_V3LEN(_a)		(sqrt(FX_SQUARE((_a)[0])+FX_SQUARE((_a)[1])+FX_SQUARE((_a)[2])))
//````````````````````````````````````````````````````````````````````````````````````````````````


#define FX_V3RAYPOINT(_pp, _origin, _dir, _t) \
	( (_pp)[0] = (_origin)[0] + (_dir)[0] * (_t),\
	  (_pp)[1] = (_origin)[1] + (_dir)[1] * (_t),\
	  (_pp)[2] = (_origin)[2] + (_dir)[2] * (_t) )

#define FX_V3LERP(_pp, _origin, _dir, _t) \
	( (_pp)[0] = (_origin)[0] + ((_dir)[0]-(_origin)[0]) * (_t),\
	  (_pp)[1] = (_origin)[1] + ((_dir)[1]-(_origin)[1]) * (_t),\
	  (_pp)[2] = (_origin)[2] + ((_dir)[2]-(_origin)[2]) * (_t) )



#define _fxMatrixInit(m, homogen)  \
	m[0][0]=1; m[0][1]=0; m[0][2]=0;\
	m[1][0]=0; m[1][1]=1; m[1][2]=0;\
	m[2][0]=0; m[2][1]=0; m[2][2]=1;\
	\
	if (homogen)\
	{\
										 m[0][3]=0;\
										 m[1][3]=0;\
										 m[2][3]=0;\
		m[3][0]=0; m[3][1]=0; m[3][2]=0; m[3][3]=1;\
	}


#define _fxMatrixInvert(dst, src)  \
	dst[0][0] = src[0][0];\
	dst[1][0] = src[0][1];\
	dst[2][0] = src[0][2];\
	dst[0][1] = src[1][0];\
	dst[1][1] = src[1][1];\
	dst[2][1] = src[1][2];\
	dst[0][2] = src[2][0];\
	dst[1][2] = src[2][1];\
	dst[2][2] = src[2][2]   


#define _fxMatrixCopy(dst, src, homogen)  \
	dst[0][0] = src[0][0];\
	dst[0][1] = src[0][1];\
	dst[0][2] = src[0][2];\
	dst[1][0] = src[1][0];\
	dst[1][1] = src[1][1];\
	dst[1][2] = src[1][2];\
	dst[2][0] = src[2][0];\
	dst[2][1] = src[2][1];\
	dst[2][2] = src[2][2];\
	\
	if (homogen)\
	{\
		dst[3][0] = src[3][0];\
		dst[3][1] = src[3][1];\
		dst[3][2] = src[3][2];\
		dst[3][3] = src[3][3];\
		dst[0][3] = src[0][3];\
		dst[1][3] = src[1][3];\
		dst[2][3] = src[2][3];\
	}

#define _fxMatrixMult(p, m1, m2, homogen)  \
	if (!(homogen))\
	{\
		p[0][0] = (double)(m1[0][0]*m2[0][0] + m1[0][1]*m2[1][0] + m1[0][2]*m2[2][0]);\
		p[0][1] = (double)(m1[0][0]*m2[0][1] + m1[0][1]*m2[1][1] + m1[0][2]*m2[2][1]);\
		p[0][2] = (double)(m1[0][0]*m2[0][2] + m1[0][1]*m2[1][2] + m1[0][2]*m2[2][2]);\
	\
		p[1][0] = (double)(m1[1][0]*m2[0][0] + m1[1][1]*m2[1][0] + m1[1][2]*m2[2][0]);\
		p[1][1] = (double)(m1[1][0]*m2[0][1] + m1[1][1]*m2[1][1] + m1[1][2]*m2[2][1]);\
		p[1][2] = (double)(m1[1][0]*m2[0][2] + m1[1][1]*m2[1][2] + m1[1][2]*m2[2][2]);\
	\
		p[2][0] = (double)(m1[2][0]*m2[0][0] + m1[2][1]*m2[1][0] + m1[2][2]*m2[2][0]);\
		p[2][1] = (double)(m1[2][0]*m2[0][1] + m1[2][1]*m2[1][1] + m1[2][2]*m2[2][1]);\
		p[2][2] = (double)(m1[2][0]*m2[0][2] + m1[2][1]*m2[1][2] + m1[2][2]*m2[2][2]);\
	}\
	else\
	{\
		p[0][0] = (double)(m1[0][0]*m2[0][0] + m1[0][1]*m2[1][0] + m1[0][2]*m2[2][0] + m1[0][3]*m2[3][0]);\
		p[0][1] = (double)(m1[0][0]*m2[0][1] + m1[0][1]*m2[1][1] + m1[0][2]*m2[2][1] + m1[0][3]*m2[3][1]);\
		p[0][2] = (double)(m1[0][0]*m2[0][2] + m1[0][1]*m2[1][2] + m1[0][2]*m2[2][2] + m1[0][3]*m2[3][2]);\
		p[0][3] = (double)(m1[0][0]*m2[0][3] + m1[0][1]*m2[1][3] + m1[0][2]*m2[2][3] + m1[0][3]*m2[3][3]);\
	\
		p[1][0] = (double)(m1[1][0]*m2[0][0] + m1[1][1]*m2[1][0] + m1[1][2]*m2[2][0] + m1[1][3]*m2[3][0]);\
		p[1][1] = (double)(m1[1][0]*m2[0][1] + m1[1][1]*m2[1][1] + m1[1][2]*m2[2][1] + m1[1][3]*m2[3][1]);\
		p[1][2] = (double)(m1[1][0]*m2[0][2] + m1[1][1]*m2[1][2] + m1[1][2]*m2[2][2] + m1[1][3]*m2[3][2]);\
		p[1][3] = (double)(m1[1][0]*m2[0][3] + m1[1][1]*m2[1][3] + m1[1][2]*m2[2][3] + m1[1][3]*m2[3][3]);\
	\
		p[2][0] = (double)(m1[2][0]*m2[0][0] + m1[2][1]*m2[1][0] + m1[2][2]*m2[2][0] + m1[2][3]*m2[3][0]);\
		p[2][1] = (double)(m1[2][0]*m2[0][1] + m1[2][1]*m2[1][1] + m1[2][2]*m2[2][1] + m1[2][3]*m2[3][1]);\
		p[2][2] = (double)(m1[2][0]*m2[0][2] + m1[2][1]*m2[1][2] + m1[2][2]*m2[2][2] + m1[2][3]*m2[3][2]);\
		p[2][3] = (double)(m1[2][0]*m2[0][3] + m1[2][1]*m2[1][3] + m1[2][2]*m2[2][3] + m1[2][3]*m2[3][3]);\
	\
		p[3][0] = (double)(m1[3][0]*m2[0][0] + m1[3][1]*m2[1][0] + m1[3][2]*m2[2][0] + m1[3][3]*m2[3][0]);\
		p[3][1] = (double)(m1[3][0]*m2[0][1] + m1[3][1]*m2[1][1] + m1[3][2]*m2[2][1] + m1[3][3]*m2[3][1]);\
		p[3][2] = (double)(m1[3][0]*m2[0][2] + m1[3][1]*m2[1][2] + m1[3][2]*m2[2][2] + m1[3][3]*m2[3][2]);\
		p[3][3] = (double)(m1[3][0]*m2[0][3] + m1[3][1]*m2[1][3] + m1[3][2]*m2[2][3] + m1[3][3]*m2[3][3]);\
	}


#define _fxMatrixXform(_x, _v, m, trans)  \
	(_x)[0] = (double)((_v)[0]*m[0][0] + (_v)[1]*m[1][0] + (_v)[2]*m[2][0]);\
	(_x)[1] = (double)((_v)[0]*m[0][1] + (_v)[1]*m[1][1] + (_v)[2]*m[2][1]);\
	(_x)[2] = (double)((_v)[0]*m[0][2] + (_v)[1]*m[1][2] + (_v)[2]*m[2][2]);\
	\
	if (trans)\
	{\
		(_x)[0] += (m[3][0]);\
		(_x)[1] += (m[3][1]);\
		(_x)[2] += (m[3][2]);\
	}

#endif //__MESSIAH_MATH

