/*=============================================================================
	Copyright (c) 1999-2003 pmG Worldwide, LLC.

	File:	messiah_plugin.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 05/09/03 - LM
			- added VARIABLE, GROUP, & ARMATURE

	------------------------------------------------------------
	PLUGIN CONFIGURATION
	------------------------------------------------------------
	This file encapsulates everything necessary to create an "empty"
	messiah plugin.  A messiah plugin MUST have _pluginEntry.  This
	is the primary entry point that designates a messiah plugin:

	EX1: messiah Entry Point Function

		_pluginEntry
		{
			-
			-
			-
			return FX_PLUGIN_OK;
		}


	Once that "wrapper" is in place, you'll need to do some setup.
	Fortunately, this header contains macros that you can use to
	perform easy plugin setup.  For example, if you were creating a
	plugin that only registers functions for scripts & expressions:
	
	EX2: Plugin Configuration

		_pluginEntry
		{
			_MESSIAH_PLUGIN_FUNCS("Your Plugin's Name");
			-
			-
			-
			return FX_PLUGIN_OK;
		}


	In EX2 we're using _MESSIAH_PLUGIN_FUNCS that configures the
	plugin for publishing functions for use in the expressions, 
	scripts, and commands.  However, the majority of times you will
	probably be writing effects, tools, shaders, and/or other modules.

	EX3: Plugin Configuration for Effects

		_pluginEntry
		{
			_MESSIAH_PLUGIN_EFFECTS("Your Plugin's Name");
			-
			-
			-
			return FX_PLUGIN_OK;
		}
	
	In the above example, the _MESSIAH_PLUGIN_EFFECTS macro configures
	the plugin for effect modules.  But what if you wanted your plugin
	to contain functions as well as effects?  The configuration macros
	essentially just set API component flags.  All you have to do is
	set the desired components...

	EX4: API Component Requests

		_pluginEntry
		{
			_API_FUNCTION;
			_API_ARG;
			_MESSIAH_PLUGIN_EFFECTS("Your Plugin's Name");
			-
			-
			-
			return FX_PLUGIN_OK;
		}
	
	
	As you can see, there's not much to it.  You simply call the
	relevant component macro before calling the configuration macro.
	
	If you need to include a lot of different components and just
	want to include everything in one shot, there are a couple of
	macros to accomplish this.


	EX5: Requesting All Components

		_pluginEntry
		{
			_MESSIAH_PLUGIN_ALL("Your Plugin's Name");
			-
			-
			-
			return FX_PLUGIN_OK;
		}
	

	Using _MESSIAH_PLUGIN_ALL will get every messiah component.
	Although it may not be very efficient since you'll undoubtedly be
	reteiving components that you don't need, but it's very convenient;
	you don't have to worry about which components to retreive, you
	can use any function in the API.  However...


	------------------------------------------------------------
	PROGRAM COMPATIBILITY
	------------------------------------------------------------
	Because _MESSIAH_PLUGIN_ALL gets *all* messiah API components,
	any plugin that uses it will be	incompatible with the entire
	messiah product line. For example, if you've configured with
	_MESSIAH_PLUGIN_ALL, your plugin will not load into
	messiah:animate.  This is because you're attempting to retreive
	messiah:studio-specific components.  This is to be expected if
	your plugin includes shader modules.  However if you want your
	plugin to work with messiah:animate AND messiah:studio, you'll
	need to use a different configuration macro...

	EX6: messiah:animate AND messiah:studio Compatibility

		_pluginEntry
		{
			_MESSIAH_ANIMATE_STUDIO("Your Plugin's Name");
			-
			-
			-
			return FX_PLUGIN_OK;
		}
	

	
	------------------------------------------------------------
	MODULE REGISTRATION
	------------------------------------------------------------
	Once you've completed the appropriate configuration, you can
	begin registering your modules using the appropriate API calls.

	EX7: Registering Function Modules

		_pluginEntry
		{
			_MESSIAH_PLUGIN_FUNCS("Your Plugin's Name");

			fxFunctionDescription("Plugin API Test Function.", 0);
			fxFunctionArgInt("messageID", 0, NULL);
			fxFunctionRegister("TestAPI", &APItest, &HelloMessage, FX_ARG_INT, FX_FUNC_SCRIPTSONLY);

			fxFunctionDescription("Second Function.", 0);
			fxFunctionArgInt("Object", FX_NULLID, "CurObject()");
			fxFunctionArgDouble("Value", 0.0, NULL);
			fxFunctionRegister("ObjFunc", &ExpObjFunc, NULL, FX_ARG_DOUBLE, 0);
			-
			-
			-
			return FX_PLUGIN_OK;
		}

	In this same way, you can register all the elements tha would make
	up your plugin, e.g. Functions, Commands, Effects, Shaders, etc.

=============================================================================*/


#ifndef	__MESSIAH_PLUGIN
#define __MESSIAH_PLUGIN



//-----------------------------------------------------------------------------
#ifndef	__MESSIAH_HIDE_TYPES

	//Convenient macros for declaring globals in your files....................
	#ifdef _MAINPLUGIN_FILE
		#ifdef __cplusplus
			#define FX_API(t, v)		extern "C" t (*v)
			#define FX_VAR(t, v)		extern "C" t v
			#define FX_VAR_I(t, v, i)	extern "C" t v = i
		#else
			#define FX_API(t, v)		t (*v)
			#define FX_VAR(t, v)		t v
			#define FX_VAR_I(t, v, i)	t v = i
		#endif
	#else
		#ifdef __cplusplus
			#define FX_API(t, v)		extern "C" t (*v)
			#define FX_VAR(t, v)		extern "C" t v
			#define FX_VAR_I(t, v, i)	extern "C" t v
		#else
			#define FX_API(t, v)		extern t (*v)
			#define FX_VAR(t, v)		extern t v
			#define FX_VAR_I(t, v, i)	extern t v
		#endif
	#endif



	#define FX_FUNC					FX_VAR

	//Macros for getting api component functions...............................
	#define	FX_APIFUNC(g, t)		g = fxGetAPIFunc(#g, t)
	#define FX_PTR(g)				if (!(FX_APIFUNC(g, messiah_apicomponent)))\
									{\
										if (fxMessageSend)\
											fxMessageSend(FX_NULLID, plugin_name,\
												FX_MESSAGE_POST|FX_MESSAGE_ERROR, #g " could not be found!", NULL);\
										return FX_FALSE;\
									}

#endif	//__MESSIAH_HIDE_TYPES
//`````````````````````````````````````````````````````````````````````````````




//Plugin Entry/Exit function def macros----------------------------------------
#define _pluginEntry	_MESSIAH_EXPORT\
						FXint messiahEntry(FXchar pname[256], FXvoid *api_func, FXint api, FXint program, const FXchar *rev)

#define _pluginExit		_MESSIAH_EXPORT\
						FXvoid messiahExit(FXvoid)

#define _pluginAPI		_MESSIAH_EXPORT\
						FXint messiahAPI(FXvoid)
//`````````````````````````````````````````````````````````````````````````````


//Plugin Return Codes----------------------------------------------------------
#define FX_PLUGIN_OK				0 //Everything proceded according to plan
#define FX_PLUGIN_BADAPI			1 //Wrong API version
#define FX_PLUGIN_BADAPIREV			2 //Wrong API revision
#define FX_PLUGIN_BADAPIFUNC		4 //Couldn't retrieve a required API function
#define FX_PLUGIN_BADPROGRAM		5 //Wrong messiah program
#define FX_PLUGIN_BADVERSION		6 //Can't support a specific version of the program
#define FX_PLUGIN_BADREVISION		7 //Can't support a specific revision of the program
#define FX_PLUGIN_BADBUILD			8 //Can't support a specific build of the program
//`````````````````````````````````````````````````````````````````````````````


//Plugin Definition & Verification---------------------------------------------
#define _API_CHECKVERSION(v)		if (_GET_MAJORREV(api)<v) return FX_PLUGIN_BADAPI
#define _API_CHECKREV(v)			if (api<v) return FX_PLUGIN_BADAPIREV
#define _API_CHECKPROGRAM(p)		if (_GET_PROGRAM(program)!=p) return FX_PLUGIN_BADPROGRAM
#define _API_GETAPIFUNC				fxGetAPIFunc = api_func
#define _API_PLUGIN_NAME(n)			strncpy(pname, n, 255);\
									strcpy(plugin_name, pname)
//`````````````````````````````````````````````````````````````````````````````


//API Component type flags-----------------------------------------------------
#define	FX_APIFUNCS_OBJECT		(1<<0)
#define	FX_APIFUNCS_MESH		(1<<1)
#define	FX_APIFUNCS_MOTION		(1<<2)
#define	FX_APIFUNCS_INTERFACE	(1<<3)
#define	FX_APIFUNCS_EFFECT		(1<<4)
#define	FX_APIFUNCS_SHADER		(1<<5)
#define	FX_APIFUNCS_IMAGE		(1<<6)
#define	FX_APIFUNCS_FUNCTION	(1<<7)
#define	FX_APIFUNCS_COMMAND		(1<<8)
#define	FX_APIFUNCS_MESSAGE		(1<<9)
#define	FX_APIFUNCS_ARG			(1<<10)
#define	FX_APIFUNCS_DATA		(1<<11)
#define	FX_APIFUNCS_FILE		(1<<12)
#define	FX_APIFUNCS_STRING		(1<<13)
#define	FX_APIFUNCS_TIME		(1<<14)
#define	FX_APIFUNCS_MODULE		(1<<15)
#define	FX_APIFUNCS_EXPFUNC		(1<<16)
#define	FX_APIFUNCS_VARIABLE	(1<<17)
#define	FX_APIFUNCS_GROUP		(1<<18)
#define	FX_APIFUNCS_ARMATURE	(1<<19)
#define	FX_APIFUNCS_ACTION		(1<<20)
#define	FX_APIFUNCS_TOOLTYPE	(1<<21)
#define	FX_APIFUNCS_UTILITY		(1<<22)
#define	FX_APIFUNCS_CUSTOM		(1<<31)

#define	FX_APIFUNCS_ALL			0x7fffffff
//`````````````````````````````````````````````````````````````````````````````


//API Component Request & Retrieval--------------------------------------------
#define	_API_OBJECT					messiah_apicomponent_flags |= FX_APIFUNCS_OBJECT
#define	_API_MESH					messiah_apicomponent_flags |= FX_APIFUNCS_MESH
#define	_API_MOTION					messiah_apicomponent_flags |= FX_APIFUNCS_MOTION
#define	_API_INTERFACE				messiah_apicomponent_flags |= FX_APIFUNCS_INTERFACE
#define	_API_EFFECT					messiah_apicomponent_flags |= FX_APIFUNCS_EFFECT
#define	_API_SHADER					messiah_apicomponent_flags |= FX_APIFUNCS_SHADER
#define	_API_IMAGE					messiah_apicomponent_flags |= FX_APIFUNCS_IMAGE
#define	_API_FUNCTION				messiah_apicomponent_flags |= FX_APIFUNCS_FUNCTION
#define	_API_COMMAND				messiah_apicomponent_flags |= FX_APIFUNCS_COMMAND
#define	_API_MESSAGE				messiah_apicomponent_flags |= FX_APIFUNCS_MESSAGE
#define	_API_ARG					messiah_apicomponent_flags |= FX_APIFUNCS_ARG
#define	_API_DATA					messiah_apicomponent_flags |= FX_APIFUNCS_DATA
#define	_API_FILE					messiah_apicomponent_flags |= FX_APIFUNCS_FILE
#define	_API_STRING					messiah_apicomponent_flags |= FX_APIFUNCS_STRING
#define	_API_TIME					messiah_apicomponent_flags |= FX_APIFUNCS_TIME
#define	_API_MODULE					messiah_apicomponent_flags |= FX_APIFUNCS_MODULE
#define	_API_EXPFUNC				messiah_apicomponent_flags |= FX_APIFUNCS_EXPFUNC
#define	_API_VARIABLE				messiah_apicomponent_flags |= FX_APIFUNCS_VARIABLE
#define	_API_GROUP					messiah_apicomponent_flags |= FX_APIFUNCS_GROUP
#define	_API_ARMATURE				messiah_apicomponent_flags |= FX_APIFUNCS_ARMATURE
#define	_API_ACTION					messiah_apicomponent_flags |= FX_APIFUNCS_ACTION
#define	_API_TOOLTYPE				messiah_apicomponent_flags |= FX_APIFUNCS_TOOLTYPE
#define	_API_UTILITY				messiah_apicomponent_flags |= FX_APIFUNCS_UTILITY
#define	_API_ALL					messiah_apicomponent_flags =  FX_APIFUNCS_ALL

#define _API_GETCOMPONENTS			if (!pluginGetAPIComponents(messiah_apicomponent_flags)) return FX_PLUGIN_BADAPIFUNC
//`````````````````````````````````````````````````````````````````````````````



//Easy Plugin Configuration Macros-----------------------------------------------------
#define _MESSIAH_PLUGIN_ANIMATE(n)	_API_CHECKVERSION(_MESSIAH_API_VERSION);\
									_API_PLUGIN_NAME(n);\
									_API_GETAPIFUNC;\
									_API_MESSAGE;\
									_API_MODULE;\
									_API_ARG;\
									_API_GETCOMPONENTS

#define _MESSIAH_PLUGIN_STUDIO(n)	_API_CHECKPROGRAM(_MESSIAH_PROGRAM_STUDIO);\
									_MESSIAH_PLUGIN_ANIMATE(n)

#define _MESSIAH_PLUGIN_FUNCS(n)	_API_PLUGIN_NAME(n);\
									_API_GETAPIFUNC;\
									_API_MESSAGE;\
									_API_FUNCTION;\
									_API_ARG;\
									_API_GETCOMPONENTS

#define _MESSIAH_PLUGIN_COMMANDS(n)	_API_PLUGIN_NAME(n);\
									_API_GETAPIFUNC;\
									_API_MESSAGE;\
									_API_COMMAND;\
									_API_GETCOMPONENTS

#define _MESSIAH_PLUGIN_EFFECTS(n)	_API_EFFECT;\
									_API_OBJECT;\
									_API_MESH;\
									_API_MOTION;\
									_API_INTERFACE;\
									_MESSIAH_PLUGIN_ANIMATE(n)

#define _MESSIAH_PLUGIN_SHADERS(n)	_API_SHADER;\
									_API_OBJECT;\
									_API_MESH;\
									_API_MOTION;\
									_API_INTERFACE;\
									_MESSIAH_PLUGIN_STUDIO(n)

#define _MESSIAH_PLUGIN_ALL(n)		_API_ALL;\
									_MESSIAH_PLUGIN_STUDIO(n)

#define _MESSIAH_ANIMATE_STUDIO(n)	_API_ALL;\
									messiah_apicomponent_flags&=(~FX_APIFUNCS_SHADER);\
									_MESSIAH_PLUGIN_ANIMATE(n)
//`````````````````````````````````````````````````````````````````````````````



//-----------------------------------------------------------------------------
#ifndef	__MESSIAH_HIDE_TYPES

	//required plugin name global....................
	FX_VAR_I(FXchar, plugin_name[256], " ");
	FX_VAR_I(FXint, messiah_apicomponent, 0);
	FX_VAR_I(FXint, messiah_apicomponent_flags, 0);
	FX_VAR_I(FXint, messiah_apicomponents_retrieved, 0);

	#ifdef _MAINPLUGIN_FILE
		//api version export.........................
		_pluginAPI
		{
			return _SET_API_REV(_MESSIAH_API_MAJORREV, _MESSIAH_API_MINORREV);
		}
	#endif

#endif	//__MESSIAH_HIDE_TYPES
//`````````````````````````````````````````````````````````````````````````````




#endif	//__MESSIAH_PLUGIN
