/*###################################################################################
=====================================================================================
	messiahHOST.h

	Authored by Lyle Milton - 10/23/02
	Copyright (c) 1998-2004 pmG Worldwide, LLC. All Rights Reserved.

	This is the header for the messiah Host API.  BETA 1


=====================================================================================
###################################################################################*/

#ifndef MESSIAH_HOST_DLL
#define MESSIAH_HOST_DLL

#define MESSIAH_HOSTAPI_VERSION  (0x00100001)

#define MESSIAH_SCENE_LOAD	0
#define MESSIAH_SCENE_MERGE 1
#define MESSIAH_SCENE_CLEAR 2

#define MESSIAH_ACTIVATED_SYSTEM	(1<<0)
#define MESSIAH_ACTIVATED_INTERFACE	(1<<1)

#define HOST_OBJECT_CREATE				(1<<0)
#define HOST_OBJECT_DELETE				(1<<1)
#define HOST_OBJECT_ADD_TO_HEIRARCHY	(1<<2)

#define FXobject	INT_PTR //64bit compatibility int
#define FXint		int
#define FXvoid		void
#define FXchar		char
#define FXdouble	double
#define FXfloat		float
#define FXuint		unsigned int
#define FXushort	unsigned short


//This is a node struct that messiah uses to manage each connected object,,,,,,,,,,,,,,,,,,,
typedef struct FX_NodeLog_St
{
	FXint					type;
	FXint					flags;

	FXint					ID; //this ID gets written to the file for each node
	FXint					load_ID;//load the saved ID into this member

	//These 2 pointers are really arbitrary.  You may only need to use one of them
	//but, 2 are provided for convenience to reduce the need for an additional struct.
	//===============================================================================
	FXvoid					*ref; //host instance pointer
	FXvoid					*data; //host data pointer
	//===============================================================================

	FXvoid					*element; //pointer to source host object - (the attached host object)
	FXobject				obj; //messiah object

	FXint					counter; //custom counter - do whatcha like

	struct FX_NodeLog_St	*next;
	struct FX_NodeLog_St	*prev;

}FX_NodeLog;


#define FX_NODE_REFERENCE		(1<<0)

#define FX_NODE_DISPLACEMENT	0 //displacement plugin
#define FX_NODE_MOTION			1 //motion plugin

//Node Object types,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
#define FX_NODE_GEOMETRIC		0
#define FX_NODE_NULL			1
#define FX_NODE_CAMERA			2
#define FX_NODE_LIGHT			3
//`````````````````````````````````````````````````````````````````````````````

//Node Light subtypes,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
#define FX_NLIGHT_DIRECTIONAL	0
#define FX_NLIGHT_POINT			1
#define FX_NLIGHT_SPOT			2
#define FX_NLIGHT_SPHERE		3
#define FX_NLIGHT_PANEL			4
//`````````````````````````````````````````````````````````````````````````````

//Surface params,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
#define FX_MSURF_AMB		"Ambient"		//c[r], c[g], c[b]
#define FX_MSURF_DIFF		"Diffuse"		//c[d]
#define FX_MSURF_SPEC		"Specular"		//c[s]
#define FX_MSURF_SHIN		"Shininess"		//c[s]
#define FX_MSURF_LUM		"Luminous"		//c[l]
#define FX_MSURF_TRANS		"Transparency"	//c[t]
//`````````````````````````````````````````````````````````````````````````````

//=====================================================================================
//Begin HOST Specific  ================================================================
//=====================================================================================

#ifdef MESSIAH_HOST_MODULE


#define MESSIAH_MODULE_NAME		"messiahHOST.dll"
#define TOTAL_NUM_CHANNELS	32


#ifdef HOST_MODULE
	#define HOSTVAR(t,v)	t *v
	#define HOSTFUNC(t, f) t (*f)
#else
	#define HOSTVAR(t,v)	extern t *v
	#define HOSTFUNC(t, f) extern t (*f)
#endif


//Important for using C++ functions & variables in messiah,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
//
//Usage:
// _EXPORTVAR(FXint) variable;
// _EXPORTVAR(FXdouble) junk = 0.0;
// _EXPORTVAR(FXvoid*) hostFunc(FXvoid *data, FXint val){ }
#if defined(__cplusplus)

	#define __EXPORTVAR(t)	extern "C" t

#else

	#define __EXPORTVAR(t)	t

#endif



//These are the functions that the host calls to establish and manage the messiah connection
#if defined(__cplusplus)
extern "C"
{
#endif

/*===============================================================================
MESSIAH GLOBALS
	This function is used to retrieve the following globals from messiah.  It
	should normally be called from within a host-supplied "hostGlobalsGetAll"
	(or some suched named) function.

	EX:
		FXvoid hostGlobalsGetAll(FXvoid)
		{
			messiahStart = messiahGlobal("messiahStart");
			messiahInterface = messiahGlobal("messiahInterface");
			.
			.
			.
		}
	
	Your hostGlobalsGetAll() function should be called right after loading the 
	messiahHOST.dll.
===============================================================================*/

	HOSTFUNC(FXvoid*, messiahGlobal)(FXchar global_name[]);


/*===============================================================================
MESSIAH SYSTEM MANAGEMENT
	These are the necessary pointers to activate, manage, and shut down messiah.
	Without these functions there can be no messiah-to-HOST connection.
===============================================================================*/

	HOSTVAR(FXdouble, messiah_current_frame);

	HOSTVAR(FXint, messiah_Activated);
	HOSTFUNC(FXint, messiahStart)(FXchar host_name[], FXchar module_name[], FXvoid *(*globalfunc)(FXchar *), FXint api_version);
	HOSTFUNC(FXint, messiahInterface)(FXvoid);
	HOSTFUNC(FXvoid, messiahShutdown)(FXvoid);
	HOSTFUNC(FXvoid, messiahCheckTime)(FXdouble time, FXint flags);
	HOSTFUNC(FXvoid, messiahUpdate)(FXint flags);


/*===============================================================================
MESSIAH SCENE FILE SYSTEM
	messiah can embed its data directly into the host scene file.  But rather
	than saving specific object/effect info on a per basis, it can embed a
	complete .fxs scene file.  This is done to provide a single and direct system
	for messiah to work with all hosts.  This also reduces the necessity for
	messiah scene file management since, from the user's perspective, there is no
	messiah scene file.

	However, it's still possible to use a "linked scene" mechanism.
===============================================================================*/

	HOSTFUNC(FXint, messiahSceneFileCreate)(FXint flags);
	HOSTFUNC(FXint, messiahSceneFileWriteLine)(FXchar *buff, FXint flags);
	HOSTFUNC(FXint, messiahSceneFileClose)(FXint flags);
	HOSTFUNC(FXchar*, messiahSceneFileCreateBuff)(FXvoid);
	HOSTFUNC(FXint, messiahSceneFileLoadBuff)(FXchar *buff);
	HOSTFUNC(FXint, messiahSceneLoad)(FXint	merge);
	HOSTFUNC(FXvoid, messiahSceneClear)(FXint flags);
	HOSTFUNC(FXint, messiahSceneFileFilterBuff)(FXchar **buff, FXchar *filter);
	HOSTFUNC(FXvoid, messiahSceneFileClearBuff)(FXchar *buff);


/*===============================================================================
NODE LOGGING SYSTEM
    A node is simply a container for a single messiah-to-HOST object connection.
	This is how we're able to know which objects need updating as well as
	performing other operations.

	Nodes can be persistant or transient.  For example, 3dsmax gives you constant
	pointers to internal data so the nodes can be freed at any time, and re-
	created.  This is good because the log can be destroyed before doing a scene
	merge, each reference can be added to the log, and upon "scene merge end"
	the objects can be reattached.  The merge log can then be destoyed, and re-
	created for the entire scene.  In this way, we can always get a reference to
	all the data we need for any operation.

	However in LightWave, the node IS your data.  If you destroy a node before
	LightWave tells you to, you'll get a crash.  So, the nodes must persist and
	you should never call nodeClearAllElements.  But, it's safe to use the node
	management functions.
===============================================================================*/

	HOSTVAR(FXint, num_nodes);
	HOSTVAR(FX_NodeLog, node_log);

	HOSTFUNC(FX_NodeLog*, nodeLogElement)(FXvoid *ref, FXvoid *element, FXvoid *data, FXobject);
	HOSTFUNC(FX_NodeLog*, nodeGetElement)(FXvoid *ref, FXvoid *element, FXvoid *data, FXobject);
	HOSTFUNC(FX_NodeLog*, nodeGetFirst)(FXvoid);
	HOSTFUNC(FX_NodeLog*, nodeGetLast)(FXvoid);
	HOSTFUNC(FX_NodeLog*, nodeSearchElement)(FXint ID);
	HOSTFUNC(FXint, nodeClearElement)(FXvoid *node, FXint flags);
	HOSTFUNC(FXvoid, nodeClearAllElements)(FXint flags);


/*===============================================================================
NODE OBJECT SYSTEM
	These are the functions you'd use to create the object inside the messiah
	environment.  Since each host has its own object format, and some can not be
	directly parsed, this system is necessary for messiah to do its work.
===============================================================================*/

	HOSTFUNC(FXobject, nodeObjectCreate)(FXchar *name, FXint type, FXint flags);
	HOSTFUNC(FXint, nodeObjectRename)(FXobject, FXchar *name, FXint flags);
	HOSTFUNC(FXint, nodeObjectNameGet)(FXobject, FXchar *buffer, FXint buffer_size);
	HOSTFUNC(FXint, nodeObjectPathGet)(FXobject, FXchar *buffer, FXint buffer_size);

	HOSTFUNC(FXint, nodeObjectNumPoints)(FXobject);
	HOSTFUNC(FXint, nodeObjectCreatePoint)(FXobject, FXdouble x, FXdouble y, FXdouble z);
	HOSTFUNC(FXint, nodeObjectGetFilePoint)(FXobject, FXint i, FXdouble *x, FXdouble *y, FXdouble *z);
	HOSTFUNC(FXint, nodeObjectGetPoint)(FXobject, FXint i, FXdouble *x, FXdouble *y, FXdouble *z);
	HOSTFUNC(FXint, nodeObjectSetPoint)(FXobject, FXint i, FXdouble x, FXdouble y, FXdouble z);

	HOSTFUNC(FXint, nodeObjectNumSurfaces)(FXobject);
	HOSTFUNC(FXint, nodeObjectCreateSurface)(FXobject, FXchar *name);
	HOSTFUNC(FXvoid, nodeObjectSurfaceSet)(FXobject, FXint i, FXchar *param, FXvoid *data);
	HOSTFUNC(FXint, nodeObjectSurfaceGet)(FXobject, FXint i, FXchar *param, FXvoid *data);
	HOSTFUNC(FXint, nodeObjectSurfaceNameGet)(FXobject, FXint i, FXchar *buffer, FXint buffer_size);

	HOSTFUNC(FXint, nodeObjectNumPolygons)(FXobject);
	HOSTFUNC(FXvoid, nodeObjectCreatePoly)(FXobject, FXint num_points, FXint *plist, FXushort surface, FXint flags);
	HOSTFUNC(FXint, nodeObjectGetPolygon)(FXobject, FXint i, FXint *point_IDs, FXint array_size, FXint *num_points, FXint *surface_ID);

	HOSTFUNC(FXvoid, nodeObjectEnd)(FXobject, FXint flags);
	HOSTFUNC(FXint, nodeObjectRebuild)(FXobject, FXint flags);
	HOSTFUNC(FXvoid, nodeObjectAppend)(FXobject, FXint flags);

	HOSTFUNC(FXint, nodeObjectSetFilePath)(FXobject, const FXchar *name, FXint flags); //Dan 5-3-03
	HOSTFUNC(FXint, nodeObjectGetFileIndexFromLayerIndex)(FXobject, FXchar *fxname, FXint *file_index, FXint layer_index); //Dan 5-8-03
	HOSTFUNC(FXint, nodeObjectSetLayerData)(FXobject, FXint file_index, FXint layer_index, FXchar *layer_name); //Dan 5-8-03


/*===============================================================================
NODE MOTION SYSTEM
	These are the functions you'd use to get and set object motion.
===============================================================================*/

	HOSTFUNC(FXvoid, nodeObjectGetMotion)(FXobject, FXdouble channels[TOTAL_NUM_CHANNELS], FXdouble time);
	HOSTFUNC(FXvoid, nodeObjectSetMotion)(FXobject, FXdouble channels[TOTAL_NUM_CHANNELS]);
	HOSTFUNC(FXvoid, nodeObjectGetWorldPos)(FXobject, FXdouble channels[3], FXdouble time);
	HOSTFUNC(FXvoid, nodeObjectGetWorldRot)(FXobject, FXdouble channels[3], FXdouble time);
	HOSTFUNC(FXvoid, nodeObjectGetWorldScale)(FXobject, FXdouble channels[3], FXdouble time);


/*===============================================================================
NODE CAMERA SYSTEM
	These are the functions you'd use to get and set camera info.
===============================================================================*/

	HOSTFUNC(FXvoid, nodeCameraGetZoom)(FXobject, FXdouble *zoom);
	HOSTFUNC(FXvoid, nodeCameraSetZoom)(FXobject, FXdouble zoom);


/*===============================================================================
NODE LIGHT SYSTEM
	These are the functions you'd use to get and set light info.
===============================================================================*/

	HOSTFUNC(FXvoid, nodeLightGetType)(FXobject, FXint *type);
	HOSTFUNC(FXvoid, nodeLightSetType)(FXobject, FXint type);
	HOSTFUNC(FXvoid, nodeLightGetColor)(FXobject, FXfloat color[3]);
	HOSTFUNC(FXvoid, nodeLightSetColor)(FXobject, FXfloat color[3]);
	HOSTFUNC(FXvoid, nodeLightGetIntensity)(FXobject, FXdouble *intensity);
	HOSTFUNC(FXvoid, nodeLightSetIntensity)(FXobject, FXdouble intensity);


#if defined(__cplusplus)
}
#endif

#else

FXint	(*hostSetContentDirectory)(FXchar *content_buffer, FXint buff_length, FXint flags);
FXint (*hostObjectGeometryCopy)(FX_NodeLog*, FXint add_to_hierarchy);
FXint (*hostObjectGetPoints)(FX_NodeLog*, FXint reset_object);
FXint (*hostObjectRename)(FX_NodeLog*, FXint flags);
FXint (*hostObjectGetPath)(FX_NodeLog*, FXchar *object_path, FXint buff_length, FXint flags);

FXvoid* (*hostObjectLoad)(FXchar *object_path, FXint buff_length, FXint flags);
FXvoid* (*hostObjectLoadLayer)(FXchar *object_path, FXint Layer_ID, FXint buff_length, FXint flags);
FXvoid* (*hostObjectConstruct)(FXobject, FXint flags);
FXvoid* (*hostObjectAdd)(FXobject, FXchar *name, FXint type, FXint sub_type, FXint flags);
FXint (*hostAddPlugin)(FXvoid *obj_data, FXobject, FXint messiah_plugtype, FXint flags);

#endif //MESSIAH_HOST_MODULE
#endif //MESSIAH_HOST_DLL


